<?php

namespace Drupal\rapidoc_elements_field_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\link\LinkItemInterface;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;

/**
 * Plugin implementation of the 'rapidoc_elements_link_ui' formatter.
 *
 * @FieldFormatter(
 *   id = "rapidoc_elements_link_ui",
 *   label = @Translation("RapiDoc Elements UI"),
 *   description = @Translation("Formats link fields with RapiDoc Elements YAML or JSON
 *   url with a rendered RapiDoc Elements UI"), field_types = {
 *     "link"
 *   }
 * )
 */
class RapiDocElementsUILinkFormatter extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    $elements = parent::view($items, $langcode);
    $elements['#attached']['library'][] = 'rapidoc_elements_field_formatter/rapidoc_elements_field_formatter.rapidoc';
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];
    foreach ($items as $delta => $link) {
      $rapidoc_file = $this->buildUrl($link);
      $element[$delta] = [
        '#theme' => 'rapidoc_elements_ui_field_item',
        '#field_name' => $this->fieldDefinition->getName(),
        '#delta' => $delta,
        '#file_url' => $rapidoc_file,
      ];
    }
    return $element;
  }

}
